<?php
class cleanCode {
    
    /**
     * supprime les commentaires d'un script et écrit sur disque le résultat
     * 
     * @param string chemin du script PHP d'origine
     * @param string chemin du fichier de destination
     * 
     * @return bool
     */
    public static function nettoyerSource($source, $destination, $sVersion, $sType, $sFilename="")
    {
       // on vérifie si le dossier où sera écrit le code nettoyé existe,
       // et si ce n'est pas le cas, on crée la structure de dossiers nécessaire
       if ($sFilename == 'cleancode.class.php') return true;
       
       $dossier=dirname($destination);
       $mavariable = "/** 
 * Author: Thomas GILBER <thomas.gilber@ryxeo.com>
 * Date: 2011
 * Version: ".$sVersion."
 * @licence Affero GPL 3
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU Affero General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU Affero General Public License for more details.
 *
 *   You should have received a copy of the GNU Affero General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */";
       
       if(!file_exists($dossier)) cleanCode::creerDossier($dossier);
       // on nettoye ensuite le code source du script original
       $buffersrc = fopen($source, 'r');
       if (filesize($source) > 0) { 
           $contents = fread($buffersrc, filesize($source));
           switch ($sType) {
               case 'php':
                   $contents = preg_replace('#(/\*.*)(Thomas).(.*?\*/)#s', 'ENTETEAREMETRE', $contents);
                   $contents = preg_replace('#/\*.*?\*/#s', '', $contents);
                   $contents = preg_replace('#ENTETEAREMETRE#s', "$mavariable", $contents);
                   $contents = preg_replace('# //.*$#m', '', $contents);
                   $contents = preg_replace('#^//.*$#m', '', $contents); 
                   $contents = preg_replace('/^[ \t]*[\r\n]+/m', '', $contents);
                   break;
               case 'js':
                   $contents = preg_replace('#/\*.*?\*/#s', '', $contents);
                   $contents = preg_replace('# //.*$#m', '', $contents);
                   break;           
               case 'css':
                   $contents = preg_replace('#/\*.*?\*/#s', '', $contents);
                   $contents = preg_replace('/\s*([{}|:;,])\s+/', '$1', $contents);
                   $contents = preg_replace('/\s\s+(.*)/', '$1', $contents);
                   $contents = str_replace(';}', '}', $contents);
                   break;           
               case 'yml':                          
                   $contents = preg_replace('`#[^!].*$`m', '', $contents);   
                   $contents = preg_replace('/^[ \t]*[\r\n]+/m', '', $contents);             
                   break;           
           }

           $bufferdst = fopen($destination, 'w');
           //$code=php_strip_whitespace($source);

           // écriture du code obtenu et on retourne le résultat (vrai ou faux)
           //$ecriture=file_put_contents($destination, $code);
           fwrite($bufferdst, $contents);
           fclose($bufferdst);
       } else {
           cleanCode::copyfile($source, $destination);
       }
       fclose($buffersrc);
       
       //return (bool) $ecriture;
       return true;
    }
    
    /**
     * scanne tous les scripts PHP d'un dossier, supprime les commentaires d'un script et écrit sur disque le résultat
     * 
     * @param string chemin du dossier contenant les scripts PHP
     * @param string chemin du dossier de destination
     * @param string extension utilisée pour les scripts ('php' ou 'php3' ou 'php5'...)
     * 
     * @return bool
     */
    public static function nettoyerDossier($source, $destination, $version)
    {
        $aExtension = array('php', 'css', 'yml');
       // si la source est un fichier, on le nettoie
       if(!is_dir($source)) {
           return cleanCode::nettoyerSource($source, $destination, $version, '');
       } 

       // si la source est un dossier, on traite chaque fichier comportant l'extension souhaitée,
       // ainsi que tous les sous-dossiers
       $fichiers=scandir($source);
       foreach($fichiers as $script)
       {
          if($script!='.' && $script!='..')
          {
             $cheminSrc=$source.'/'.$script;
             $cheminDest=$destination.'/'.$script;
             
             // si le fichier courant est un dossier, on applique à nouveau la fonction sur ce dossier
             if(is_dir($cheminSrc)) cleanCode::nettoyerDossier($cheminSrc, $cheminDest, $version);

             // sinon, on regarde l'extension du fichier, et si c'est la bonne, on le traite
             elseif(in_array (pathinfo($script, PATHINFO_EXTENSION), $aExtension) && strpos($source, 'plugins') == false) { 
                 cleanCode::nettoyerSource($cheminSrc, $cheminDest, $version, pathinfo($script, PATHINFO_EXTENSION), $script);                  
             } else {
                 cleanCode::copyfile($cheminSrc, $cheminDest);
             }
          }
       }

       // on retourne vrai (on peut aussi imaginer renvoyer le nombre de scripts traités avec succès)
       return true;
    }
    
    public static function copyfile($cheminSrc, $cheminDest) {
       $dossier=dirname($cheminDest);
       if(!file_exists($dossier)) cleanCode::creerDossier($dossier);      
       copy($cheminSrc, $cheminDest);
    }
    /**
    * crée une structure de dossiers récursivement
    * 
    * @param string $chemin chemin du répertoire à créer
    * @return bool
    */
    public static function creerDossier($chemin)
    {
        if(is_dir($chemin)) return true;
        return cleanCode::creerDossier(dirname($chemin)) && mkdir($chemin);
    }
}

if (isset($_SERVER['argv'][1]) && isset($_SERVER['argv'][2])) {
 cleanCode::nettoyerDossier($_SERVER['argv'][1], $_SERVER['argv'][2], $_SERVER['argv'][3]);
}


